@@ -7,7 +7,7 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: ircaux.h,v 1.111 2014/01/10 13:30:40 jnelson Exp $
+ * @(#)$Id: ircaux.h,v 1.127 2014/04/11 20:38:57 jnelson Exp $
  */
 
 #ifndef _IRCAUX_H_
@@ -52,6 +52,8 @@ char *	upper 			(char *);
 char *	lower 			(char *);
 char *	sindex			(char *, const char *);
 char *	rsindex 		(char *, char *, char *, int);
+const unsigned char * cpindex 	(const unsigned char *, const unsigned char *, int, size_t *);
+const unsigned char * rcpindex 	(const unsigned char *, const unsigned char *, const unsigned char *, int, size_t *);
 ssize_t	findchar_quoted		(const char *, int);
 int	path_search 		(const char *, const char *, Filename);
 char *	double_quote 		(const char *, const char *, char *);
@@ -96,6 +98,7 @@ double 	time_diff 		(const Timeval, const Timeval);
 Timeval time_add		(const Timeval, const Timeval);
 Timeval time_subtract		(const Timeval, const Timeval);
 Timeval double_to_timeval 	(double);
+double  time_since_startup 	(Timeval t);
 const char *	plural 		(int);
 double	time_to_next_minute 	(void);
 char *	remove_trailing_spaces 	(char *, size_t *cluep);
@@ -104,7 +107,7 @@ char *	forcibly_remove_trailing_spaces (char *, size_t *);
 const char *	my_ltoa 		(long);
 const char *	intmaxtoa 		(intmax_t);
 const char *	ftoa			(double);
-char *	strformat 		(char *, const char *, ssize_t, int);
+char *	strformat 		(char *, size_t, const unsigned char *, ssize_t, int);
 char *	chop_word 		(char *);
 char *	skip_spaces		(char *);
 int	split_args		(char *, char **to, size_t);
@@ -135,7 +138,7 @@ char *	ov_strcpy		(char *, const char *);
 size_t	ccspan			(const char *, int);
 int	last_char		(const char *);
 char *	next_in_comma_list	(char *, char **);
-char *	next_in_div_list	(char *, char **, char);
+char *	next_in_div_list	(char *, char **, int);
 char *	get_userhost		(void);
 double  time_to_next_interval	(int interval);
 int	charcount		(const char *, char);
@@ -165,7 +168,8 @@ char *	endstr			(char *);
 ssize_t searchbuf		(const unsigned char *, size_t, size_t, int);
 int	remove_from_comma_list	(char *str, const char *what);
 char *	dequote_buffer		(char *str, size_t *len);
-char *	fix_string_width	(const char *, int, char, size_t);
+char *	fix_string_width	(const char *, int, int, size_t);
+int     check_xdigit 		(unsigned char digit);
 
 void	add_mode_to_str		(char *, size_t, int);
 void	remove_mode_from_str	(char *, size_t, int);
@@ -233,6 +237,7 @@ char *	substitute_string	(const char *, const char *, const char *, int, int);
 #define STRIP_ALL_OFF		1 << 9
 #define STRIP_UNPRINTABLE	1 << 10
 #define STRIP_OTHER		1 << 11
+#define STRIP_ITALIC		1 << 12
 
 extern	int	outbound_line_mangler;
 extern	int	inbound_line_mangler;
@@ -297,5 +302,35 @@ extern int my_iconv_open (iconv_t *, iconv_t *, const char *);
 #endif
 
 extern	int	recode_with_iconv (const char *from, const char *to, char **data, size_t *numbytes);
+extern	int     recode_with_iconv_t (iconv_t iref, char **data, size_t *numbytes);
+
+/***********************/
+extern	int	codepoint_numcolumns 	(int);
+extern	int	next_code_point 	(const unsigned char **);
+extern	int	partial_code_point	(const unsigned char *);
+extern	int	quick_display_column_count 	(const unsigned char *);
+extern	int	input_column_count 	(const unsigned char *);
+extern	int	ucs_to_utf8 		(u_32int_t, unsigned char *, size_t);
+extern	int	grab_codepoint 		(const unsigned char *x);
+extern	int     quick_code_point_count	(const unsigned char *str);
+extern	int     previous_code_point	(const unsigned char *, const unsigned char **);
+extern	int	quick_code_point_index	(const unsigned char *, const unsigned char *);
+
+extern	int	strext2		(unsigned char **, unsigned char *, int , int);
+extern	int     invalid_utf8str (unsigned char *utf8str);
+extern	void    create_utf8_locale (void);
+extern	int	mkupper_l	(int);
+extern	int	mklower_l	(int);
+extern	int	sanity_check_encoding (const char *);
+
+/******* recode.c ******/
+extern	void		init_recodings	(void);
+extern	const char *	find_recoding	(const char *, iconv_t *, iconv_t *);
+extern	char *		recode_message	(const char *from, const char *to, const char *comm, const char **ArgList, int which);
+extern	int     ucs_to_console (u_32int_t key, unsigned char *utf8str, size_t utf8strsiz);
+BUILT_IN_COMMAND(encoding);
+extern	const char *	outbound_recode (const char *, int, const char *, char **);
+extern	const char *	inbound_recode (const char *, int, const char *, const char *, char **);
+extern	char *	function_encodingctl (char *);
 
 #endif /* _IRCAUX_H_ */
