@@ -17,16 +17,51 @@
 #define WAIT_PROMPT_KEY         0x02
 #define WAIT_PROMPT_DUMMY	0x04
 
+typedef struct InputLine
+{
+	/* The current UTF8 input line (plain old c string */
+	unsigned char input_buffer[INPUT_BUFFER_SIZE+1];
+
+	/* The offset into input_buffer where each logical char starts */
+	int	logical_chars[INPUT_BUFFER_SIZE + 1];
+
+	/* The logical column in which each logical char lives */
+	int	logical_columns[INPUT_BUFFER_SIZE + 1];
+
+	/* Upon which logical char does the cursor sit? */
+	int	logical_cursor;
+
+	int	first_display_char;
+	int	number_of_logical_chars;
+
+	char *	input_prompt_raw;
+	unsigned char	*input_prompt;
+	int	input_prompt_len;
+	int	input_line;
+
+        unsigned char   *ind_left;
+        int     ind_left_len;
+        unsigned char   *ind_right;
+        int     ind_right_len;
+
+	int	refresh;
+	int	echo;
+}	InputLine;
+
+
 typedef struct PromptStru
 {
-	char	*prompt;
-	char	*data;
-	int	type;
-	int	echo;
-	void	(*func) (char *, char *);
+	char *		data;
+	int		type;
+	void		(*func) (char *, const char *);
+
+	InputLine *	my_input_line;
+	InputLine *	saved_input_line;
+
 	struct	PromptStru	*next;
 }	WaitPrompt;
 
+
 typedef	struct	ScreenStru
 {
 	/* List stuff and overhead */
@@ -52,10 +87,12 @@ struct	ScreenStru *next;		/* Previous screen in list */
 	int	control;		/* The control FD (to wserv) */
 	int	wserv_version;		/* The version of wserv talking to */
 
+	InputLine *	il;
+
+#if 0
 	/* Input line and prompt stuff */
 	unsigned char input_buffer[INPUT_BUFFER_SIZE+1];
 					/* Current user input for us */
-	int	buffer_pos;		/* Where on input line cursor is */
 	int	input_cursor;		/* Where the cursor is on input line */
 	int	input_visible;
 	int	input_zone_len;
@@ -70,7 +107,7 @@ struct	ScreenStru *next;		/* Previous screen in list */
         int     ind_right_len;
 
         unsigned char	saved_input_buffer[INPUT_BUFFER_SIZE+1];
-	int	saved_buffer_pos;
+#endif
 
 	WaitPrompt	*promptlist;
 
@@ -90,12 +127,13 @@ struct	ScreenStru *next;		/* Previous screen in list */
 	int	alive;
 }	Screen;
 
-	void	add_wait_prompt 	(const char *, void (*)(char *, char *), const char *, int, int);
+	void	add_wait_prompt 	(const char *, void (*)(char *, const char *), const char *, int, int);
+	void	fire_wait_prompt	(u_32int_t);
+	void	fire_normal_prompt	(const char *);
 	void	add_to_screen		(const unsigned char *);
 	void	cursor_not_in_display	(struct ScreenStru *);
 	void	cursor_in_display	(Window *);
-	void	edit_codepoint		(u_32int_t key);
-	void	edit_char		(unsigned char);
+	void	translate_user_input	(unsigned char byte);
 	int	is_cursor_in_display	(struct ScreenStru *);
 	void	repaint_window_body	(Window *);
 	void	create_new_screen	(void);
@@ -111,6 +149,9 @@ const	unsigned char *all_off			(void);
 
 	unsigned char *prepare_display2	(const unsigned char *, int, int, char, int);
 
+	void	chop_columns (unsigned char **, size_t);
+	void	chop_final_columns (unsigned char **, size_t);
+
 /* Dont do any word-wrapping, just truncate each line at its place. */
 #define PREPARE_NOWRAP	0x01
 
