@@ -1,4 +1,4 @@
-/* $EPIC: ctcp.c,v 1.59 2009/07/05 04:29:44 jnelson Exp $ */
+/* $EPIC: ctcp.c,v 1.67 2014/04/09 17:51:07 jnelson Exp $ */
 /*
  * ctcp.c:handles the client-to-client protocol(ctcp). 
  *
@@ -67,7 +67,7 @@
 #define CTCP_INLINE	2	/* Expands to an inline value */
 #define CTCP_NOLIMIT	4	/* Limit of one per privmsg. */
 #define CTCP_TELLUSER	8	/* Tell the user about it. */
-
+#define CTCP_NORECODE	16	/* Don't recode message: ctcp func handles */
 static	int 	split_CTCP (char *, char *, char *);
 
 /*
@@ -123,25 +123,32 @@ static CtcpEntry ctcp_cmd[] =
 		do_version, 	NULL },
 
 	/* Common ones to people using strong crypto */
-	{ "AESSHA256-CBC",	CTCP_AESSHA256,	CTCP_INLINE | CTCP_NOLIMIT,
+	{ "AESSHA256-CBC",	CTCP_AESSHA256,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
 		"transmit aes256-cbc ciphertext using a sha256 key",
 		do_crypto, 	do_crypto },
-	{ "AES256-CBC",		CTCP_AES256,	CTCP_INLINE | CTCP_NOLIMIT,
+	{ "AES256-CBC",		CTCP_AES256,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
 		"transmit aes256-cbc ciphertext",
 		do_crypto, 	do_crypto },
-	{ "CAST128ED-CBC",	CTCP_CAST5, 	CTCP_INLINE | CTCP_NOLIMIT,
+	{ "CAST128ED-CBC",	CTCP_CAST5, 	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
 		"transmit cast5-cbc ciphertext",
 		do_crypto, 	do_crypto},
-	{ "BLOWFISH-CBC",	CTCP_BLOWFISH,	CTCP_INLINE | CTCP_NOLIMIT,
+	{ "BLOWFISH-CBC",	CTCP_BLOWFISH,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
 		"transmit blowfish-cbc ciphertext",
 		do_crypto, 	do_crypto },
-	{ "FISH",		CTCP_FISH,	CTCP_INLINE | CTCP_NOLIMIT,
+	{ "FISH",		CTCP_FISH,	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
 		"transmit FiSH (blowfish-ecb with sha256'd key) ciphertext",
 		do_crypto, 	do_crypto },
-	{ "SED",		CTCP_SED, 	CTCP_INLINE | CTCP_NOLIMIT,
+	{ "SED",		CTCP_SED, 	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
 		"transmit simple_encrypted_data ciphertext",
 		do_crypto, 	do_crypto },
-	{ "SEDSHA",		CTCP_SEDSHA, 	CTCP_INLINE | CTCP_NOLIMIT,
+	{ "SEDSHA",		CTCP_SEDSHA, 	
+			CTCP_INLINE | CTCP_NOLIMIT | CTCP_NORECODE,
 		"transmit simple_encrypted_data ciphertext using a sha256 key",
 		do_crypto, 	do_crypto },
 
@@ -226,10 +233,23 @@ CTCP_HANDLER(do_crypto)
 	if (!key || !ret) {
 		sed = 2;
 		malloc_strcpy(&ret2, "[ENCRYPTED MESSAGE]");
-	} else if (!*ret) {
-		sed = 2;
-		malloc_strcpy(&ret2, "[ENCRYPTED MESSAGE - BAD KEY?]");
 	} else {
+		char *extra = NULL;
+
+		/* We might not need to recode since do_ctcp now does it */
+		/* We just need to decrypt */
+#if 0
+		/*
+		 * We must recode to UTF8
+		 */
+                inbound_recode(from, from_server, to, ret, &extra);
+		if (extra)
+		{
+			new_free(&ret);
+			ret = extra;
+		}
+#endif
+
 		/* 
 		 * There might be a CTCP message in there,
 		 * so we see if we can find it.
@@ -238,6 +258,12 @@ CTCP_HANDLER(do_crypto)
 			ret2 = malloc_strdup(do_ctcp(from, to, ret));
 		else if (get_server_doing_notice(from_server))
 			ret2 = malloc_strdup(do_notice_ctcp(from, to, ret));
+		else
+		{
+			ret2 = ret;
+			ret = NULL;
+		}
+
 		sed = 1;
 	}
 
@@ -567,7 +593,7 @@ CTCP_HANDLER(do_ping_reply)
 
 	tsec = t.tv_sec - orig;
 
-	if ((ptr = sindex(cmd, " .")))
+	if ((ptr = strchr(cmd, ' ')) || (ptr = strchr(cmd, '.')))
 	{
 		*ptr++ = 0;
 		tusec = t.tv_usec - my_atol(ptr);
@@ -625,6 +651,7 @@ char *	do_ctcp (const char *from, const char *to, char *str)
 	int	allow_ctcp_reply = 1;
 static	time_t	last_ctcp_parsed = 0;
 	int	l;
+	char *	extra = NULL;
 
 	int delim_char = charcount(str, CTCP_DELIM_CHAR);
 
@@ -746,6 +773,7 @@ static	time_t	last_ctcp_parsed = 0;
 			 */
 			if (do_hook(CTCP_REQUEST_LIST, "%s %s %s %s",
 				from, to, ctcp_command, ctcp_argument))
+			{
 			    if (do_hook(CTCP_LIST, "%s %s %s %s", from, to, 
 						ctcp_command, ctcp_argument))
 			    {
@@ -754,12 +782,23 @@ static	time_t	last_ctcp_parsed = 0;
 					*ctcp_argument ? ": " : empty_string, 
 					ctcp_argument);
 			    }
+			}
 			time(&last_ctcp_parsed);
 			allow_ctcp_reply = 0;
 			pop_message_from(l);
 			continue;
 		}
 
+		if (!(ctcp_cmd[i].flag & CTCP_NORECODE))
+		{
+		   /*
+		    * We must recode to UTF8
+		    */
+		   inbound_recode(from, from_server, to, ctcp_argument, &extra);
+		   if (extra)
+			ctcp_argument = extra;
+		}
+
 		/* 
 		 * We did find it.  Acknowledge it.
 		 */
@@ -818,6 +857,7 @@ static	time_t	last_ctcp_parsed = 0;
 					ctcp_argument);
 		    }
 		}
+		new_free(&extra);
 		new_free(&ptr);
 		pop_message_from(l);
 	}
@@ -985,6 +1025,8 @@ void	send_ctcp (int type, const char *to, int datatag, const char *format, ...)
 		*putbuf2;
 	int	len;
 	int	l;
+	const char *pb;
+	char	*extra = NULL;
 
 	/* Make sure that the final \001 doesnt get truncated */
 	if ((len = IRCD_BUFFER_SIZE - (12 + strlen(to))) <= 0)
@@ -999,13 +1041,17 @@ void	send_ctcp (int type, const char *to, int datatag, const char *format, ...)
 		vsnprintf(putbuf, BIG_BUFFER_SIZE, format, args);
 		va_end(args);
 
+		pb = outbound_recode(to, from_server, putbuf, &extra);
+
 		do_hook(SEND_CTCP_LIST, "%s %s %s %s", 
 				ctcp_type[type], to, 
-				ctcp_cmd[datatag].name, putbuf);
+				ctcp_cmd[datatag].name, pb);
 		snprintf(putbuf2, len, "%c%s %s%c", 
 				CTCP_DELIM_CHAR, 
-				ctcp_cmd[datatag].name, putbuf, 
+				ctcp_cmd[datatag].name, pb, 
 				CTCP_DELIM_CHAR);
+
+		new_free(&extra);
 	}
 	else
 	{
@@ -1022,7 +1068,7 @@ void	send_ctcp (int type, const char *to, int datatag, const char *format, ...)
 	putbuf2[len - 2] = CTCP_DELIM_CHAR;
 	putbuf2[len - 1] = 0;
 
-	send_text(from_server, to, putbuf2, ctcp_type[type], 0);
+	send_text(from_server, to, putbuf2, ctcp_type[type], 0, 1);
 	pop_message_from(l);
 }
 
@@ -1076,3 +1122,11 @@ static int split_CTCP (char *raw_message, char *ctcp_dest, char *after_ctcp)
 	return 0;		/* All done! */
 }
 
+void    help_topics_ctcp (FILE *f)
+{
+        int     x;                                                              
+
+        for (x = 0; ctcp_cmd[x].name; x++)                            
+                fprintf(f, "ctcp %s\n", ctcp_cmd[x].name);
+}
+
