@@ -1,4 +1,4 @@
-/* $EPIC: irc.c,v 1.1374 2014/01/16 19:31:24 jnelson Exp $ */
+/* $EPIC: irc.c,v 1.1450 2014/04/12 15:14:15 jnelson Exp $ */
 /*
  * ircII: a new irc client.  I like it.  I hope you will too!
  *
@@ -40,19 +40,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC5-1.1.7";
-const char useful_info[] = "epic5 1 1 7";
+const char irc_version[] = "EPIC5-1.1.10";
+const char useful_info[] = "epic5 1 1 10";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20140116";
+const char internal_version[] = "20140413";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 1705;
+const unsigned long	commit_id = 1781;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -60,7 +60,7 @@ const unsigned long	commit_id = 1705;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Kainotophobia";
+const char ridiculous_version_name[] = "Desquamation";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -395,7 +395,7 @@ The program will now terminate.						\n", irc_version, commit_id, internal_versi
  * the right response to our question.  If the response was affirmative, the
  * user gets booted from irc.  Otherwise, life goes on. 
  */
-static void quit_response (char *dummy, char *ptr)
+static void	quit_response (char *dummy, const char *ptr)
 {
 	int	len;
 
@@ -934,7 +934,7 @@ static	int		level = 0,
 			get_time(&now);
 			if (cntl_c_hit)		/* SIGINT is useful */
 			{
-				edit_char('\003');
+				translate_user_input('\003');
 				cntl_c_hit = 0;
 			}
 			else if (errno != EINTR) /* Deal with EINTR */
@@ -982,9 +982,11 @@ static	int		level = 0,
 	/* Move the cursor back to the input line */
 	cursor_to_input();
 
+#if 0
 #ifdef __GNUC__
 	/* GCC wants us to do this to reclaim any alloca()ed space */
 	alloca(0);
+#endif
 #endif
 
 	/* Release this io() accounting level */
@@ -1013,7 +1015,7 @@ static void check_password (void)
 #else
 	fprintf(stderr, "Passwd: ");
 	fgets(input_passwd, INPUT_PASSWD_LEN - 1, stdin);
-	chop(input_passwd);
+	chop(input_passwd, 1);
 #endif
 	if (strcmp(input_passwd, PASSWORD))
 		execl(SPOOF_PROGRAM, SPOOF_PROGRAM, NULL);
@@ -1171,6 +1173,9 @@ void    load_ircrc (void)
 int 	main (int argc, char *argv[])
 {
 	setlocale(LC_ALL, "");
+	setlocale(LC_NUMERIC, "C");
+
+	create_utf8_locale();
 	setvbuf(stdout, NULL, _IOLBF, 1024);
 #ifdef SOCKS
 	SOCKSinit(argv[0]);
@@ -1186,6 +1191,7 @@ int 	main (int argc, char *argv[])
 
 	init_levels();
 	init_transforms();
+	init_recodings();
 	init_variables_stage1();
 	parse_args(argc, argv);
 	init_binds();
@@ -1285,7 +1291,7 @@ int 	main (int argc, char *argv[])
 
 	load_ircrc();
 
-	set_input(empty_string);
+	init_input();
 
 	if (dont_connect)
 		display_server_list();		/* Let user choose server */
