@@ -1,4 +1,4 @@
-/* $EPIC: status.c,v 1.80 2014/01/10 13:30:41 jnelson Exp $ */
+/* $EPIC: status.c,v 1.85 2014/03/21 12:58:12 jnelson Exp $ */
 /*
  * status.c: handles the status line updating, etc for IRCII 
  *
@@ -125,11 +125,11 @@ static	char	*notify_format 		= (char *) 0;
 static void	init_status	(void);
 
 /*
- * Status updates are not permitted until we are connected to a server.
- * Got_initial_version_28 will set this to 1 when we first get connected
- * to a server.
+ * Status updates are not permitted while we are doing some sensitive
+ * operations (like creating a window) to avoid null derefs
  */
-int     status_updates_permitted = 0;
+	int     status_updates_permitted = 0;
+static	int	defered_status_updates = 0;
 
 /*
  * This is the list of possible expandos.  Note that you should not use
@@ -150,7 +150,7 @@ struct status_formats status_expandos[] = {
 { 0, 'D', status_dcc, 	        NULL, 			NULL },
 { 0, 'E', status_activity,	NULL,			NULL },
 { 0, 'F', status_notify_windows,&notify_format,		&STATUS_NOTIFY_VAR },
-{ 0, 'G', status_network,		NULL,			NULL },
+{ 0, 'G', status_network,	NULL,			NULL },
 { 0, 'H', status_hold,		NULL,			NULL },
 { 0, 'I', status_insert_mode,   NULL,			NULL },
 { 0, 'K', status_scrollback,	NULL,			NULL },
@@ -478,7 +478,6 @@ int	make_status (Window *window, Status *status)
 	unsigned char	lhs_buffer  [BIG_BUFFER_SIZE + 1];
 	unsigned char	rhs_buffer  [BIG_BUFFER_SIZE + 1];
 	const char	*func_value [MAX_FUNCTIONS];
-	unsigned char	*ptr;
 	size_t		save_size;
 	Screen	*	screen;
 	int		anything_changed = 0;
@@ -494,6 +493,7 @@ int	make_status (Window *window, Status *status)
 	{
 		debuglog("make_status(%d) -- no status updates right now",
 				window->refnum);
+		defered_status_updates++;
 		return -1;
 	}
 
@@ -507,21 +507,22 @@ int	make_status (Window *window, Status *status)
 
 	for (status_line = 0; status_line < status->number; status_line++)
 	{
-	unsigned char	lhs_fillchar[6],
-			rhs_fillchar[6],
-			*fillchar = lhs_fillchar,
-			*lhp = lhs_buffer,
-			*rhp = rhs_buffer,
-			*cp,
-			*start_rhs = 0,
+	int		fillchar;
+	unsigned char	*lhp = lhs_buffer,
+			*rhp = rhs_buffer;
+	unsigned char 	*cp,
 			*str = NULL;
+	const unsigned char *	start_rhs = 0;
 		int	pr_lhs = 0,
 			pr_rhs = 0,
 			line = 0,	/* XXX gcc4 lameness */
 			*prc = &pr_lhs, 
 			i;
+	const unsigned char *	s;
+		int	code_point;
+		int	cols;
 
-		fillchar[0] = fillchar[1] = 0;
+		fillchar = 0;
 
 		/*
 		 * Figure out which of the three status bars we're creating.
@@ -631,22 +632,31 @@ int	make_status (Window *window, Status *status)
 		 * characters.  We count all the printable characters
 		 * on both sides of the %> tag.
 		 */
-		ptr = str;
+		s = str;
 		cp = lhp;
 		lhs_buffer[0] = rhs_buffer[0] = 0;
 
-		while (*ptr)
+		while ((code_point = next_code_point(&s)))
 		{
+			/* 
+			 * If we find an invalid code point, skip it.
+			 * XXX Surely this is wrong -- revisit later.
+			 */
+			if (code_point == -1)
+			{
+				s++;
+				continue;
+			}
+
 			/*
 			 * The FIRST such %> tag is used.
 			 * Using multiple %>s is bogus.
 			 */
-			if (*ptr == '\f' && start_rhs == NULL)
+			if (code_point == '\f' && start_rhs == NULL)
 			{
-				ptr++;
-				start_rhs = ptr;
-				fillchar = rhs_fillchar;
+				start_rhs = s;
 				*cp = 0;
+
 				cp = rhp;
 				prc = &pr_rhs;
 			}
@@ -654,28 +664,27 @@ int	make_status (Window *window, Status *status)
                         /*
                          * Skip over attribute changes, not useful.
                          */
-                        else if (*ptr == '\006')
+                        else if (code_point == 6)
                         {
                                 /* Copy the next 5 values */
-                                *cp++ = *ptr++;
-                                *cp++ = *ptr++;
-                                *cp++ = *ptr++;
-                                *cp++ = *ptr++;
-                                *cp++ = *ptr++;
+                                *cp++ = 6;
+                                *cp++ = *s++;
+                                *cp++ = *s++;
+                                *cp++ = *s++;
+                                *cp++ = *s++;
                         }
 
 			/*
 			 * XXXXX This is a bletcherous hack.
 			 * If i knew what was good for me id not do this.
 			 */
-			else if (*ptr == 9)	/* TAB */
+			else if (code_point == 9)	/* TAB */
 			{
-				fillchar[0] = ' ';
-				fillchar[1] = 0;
+				fillchar = 32;	/* Fill becomes space */
+
 				do
 					*cp++ = ' ';
 				while (++(*prc) % 8);
-				ptr++;
 			}
 
 			/*
@@ -683,9 +692,20 @@ int	make_status (Window *window, Status *status)
 			 */
 			else
 			{
-				*prc += 1;
-				fillchar[0] = *cp++ = *ptr++;
-				fillchar[1] = 0;
+				unsigned char	utf8str[16];
+				const unsigned char *x;
+	
+				if (!start_rhs)
+					fillchar = code_point;
+
+				cols = codepoint_numcolumns(code_point);
+				if (cols == -1)
+					cols = 0;
+				*prc += cols;
+
+				ucs_to_utf8(code_point, utf8str, sizeof(utf8str));
+				for (x = utf8str; *x; x++)
+					*cp++ = *x;
 			}
 
 			/*
@@ -701,36 +721,33 @@ int	make_status (Window *window, Status *status)
 
 		/* What will we be filling with? */
 		if (get_int_var(STATUS_NO_REPEAT_VAR))
-		{
-			lhs_fillchar[0] = ' ';
-			lhs_fillchar[1] = 0;
-			rhs_fillchar[0] = ' ';
-			rhs_fillchar[1] = 0;
-		}
+			fillchar = 32;	/* Fill becomes space */
 
 		/*
 		 * Now if we have a rhs, then we have to adjust it.
 		 */
-		if (start_rhs)
+		/* Not attached, so don't "fix" it */
 		{
-			int numf = 0;
+			unsigned char	utf8str[16];
+			int 		numf = 0;
 
-			numf = screen->co - pr_lhs - pr_rhs -1;
-			while (numf-- >= 0)
-				strlcat(lhs_buffer, lhs_fillchar, 
-						sizeof lhs_buffer);
-		}
+			if ((cols = codepoint_numcolumns(fillchar)) < 1)
+			{
+				/* Use space as fillchar if necessary */
+				fillchar = 32;
+				cols = 1;
+			}
+			ucs_to_utf8(fillchar, utf8str, sizeof(utf8str));
 
-		/*
-		 * No rhs?  If the user wants us to pad it out, do so.
-		 */
-		else 
-		{
-			int chars = screen->co - pr_lhs - 1;
+			numf = screen->co - pr_lhs - 1;
+			if (start_rhs)
+				numf -= pr_rhs;
 
-			while (chars-- >= 0)
-				strlcat(lhs_buffer, lhs_fillchar, 
-						sizeof lhs_buffer);
+			while (numf >= 0)
+			{
+				strlcat(lhs_buffer, utf8str, sizeof lhs_buffer);
+				numf -= cols;
+			}
 		}
 
 		save_size = strlen(all_off());
@@ -789,6 +806,7 @@ int	redraw_status (Window *window, Status *status)
 	if (!status_updates_permitted)
 	{
 		debuglog("redraw_status: no status updates right now, try later");
+		defered_status_updates++;
 		return -1;
 	}
 
@@ -933,8 +951,20 @@ void	build_status	(void *stuff)
  */
 int     permit_status_update (int flag)
 {
-        int old_flag = status_updates_permitted;
+        int 	old_flag;
+
+	old_flag = status_updates_permitted;
         status_updates_permitted = flag;
+
+	/* XXX I hate this, but I just want this problem to go away. */
+	/* This is caused by doing a /window command within /on window_create */
+	if (flag && defered_status_updates)
+	{
+		update_all_status();
+		update_all_windows();
+		defered_status_updates = 0;
+	}
+
         return old_flag;
 }
 
