@@ -1,4 +1,4 @@
-/* $EPIC: term.c,v 1.27 2013/07/28 23:16:14 jnelson Exp $ */
+/* $EPIC: term.c,v 1.30 2014/03/05 14:40:56 jnelson Exp $ */
 /*
  * term.c -- termios and (termcap || terminfo) handlers
  *
@@ -716,7 +716,7 @@ void	term_inputline_putchar (unsigned char c)
 	 * we catch the very nasty 0x9b which is the escape character with
 	 * the high bit set.  
 	 */
-	if (c < 0x20 || c == 0x9b)
+	if (c < 0x20/* || c == 0x9b*/)
 	{
 		term_standout_on();
 		putchar_x((c | 0x40) & 0x7f);
@@ -915,7 +915,7 @@ int 	term_init (void)
 	/*
 	 * Default to no colors. (ick)
 	 */
-	for (i = 0; i < 16; i++)
+	for (i = 0; i < 256; i++)
 		current_term->TI_forecolors[i] = current_term->TI_backcolors[i] = "";
 
 	if (current_term->TI_bold)
@@ -939,6 +939,11 @@ int 	term_init (void)
 		current_term->TI_sgrstrs[TERM_SGR_ALTCHAR_ON - 1] = current_term->TI_smacs;
 	if (current_term->TI_rmacs)
 		current_term->TI_sgrstrs[TERM_SGR_ALTCHAR_OFF - 1] = current_term->TI_rmacs;
+	if (current_term->TI_sitm)
+		current_term->TI_sgrstrs[TERM_SGR_ITALIC_ON - 1] = current_term->TI_sitm;
+	if (current_term->TI_ritm)
+		current_term->TI_sgrstrs[TERM_SGR_ITALIC_OFF - 1] = current_term->TI_ritm;
+
 	if (current_term->TI_normal[0])
 	{
 		current_term->TI_sgrstrs[TERM_SGR_NORMAL - 1] = current_term->TI_normal;
@@ -1061,6 +1066,20 @@ int 	term_init (void)
 		current_term->TI_backcolors[i] = malloc_strdup(cbuf);
 	}
 
+	/* I don't know any other way to set 256 colors, so ..... */
+	for (i = 16; i < 256; i++)
+	{
+		char cbuf[128];
+
+		*cbuf = 0;
+		snprintf(cbuf, sizeof cbuf, "\033[38;5;%dm", i);
+		current_term->TI_forecolors[i] = malloc_strdup(cbuf);
+
+		*cbuf = 0;
+		snprintf(cbuf, sizeof cbuf, "\033[48;5;%dm", i);
+		current_term->TI_backcolors[i] = malloc_strdup(cbuf);
+	}
+
 /* Set up the terminal discipline */
 	tty_des = 0;			/* Has to be. */
 	tcgetattr(tty_des, &oldb);
@@ -1274,6 +1293,7 @@ void	term_beep (void)
 	}
 }
 
+/* XXX Deprecated */
 void	set_meta_8bit (void *stuff)
 {
 	VARIABLE *v;
@@ -1354,6 +1374,7 @@ void	term_clrscr (void)
 	}
 }
 
+#if 0
 /*
  * Move the cursor NUM spaces to the left, non-destruvtively if we can.
  */
@@ -1372,6 +1393,7 @@ void	term_left (int num)
 		while (num--)
 			tputs_x (current_term->TI_kbs);
 }
+#endif
 
 /*
  * Move the cursor NUM spaces to the right
@@ -1389,6 +1411,7 @@ void	term_right (int num)
 			tputs_x(current_term->TI_cuf1);
 }
 
+#if 0
 /*
  * term_delete (int num)
  * Deletes NUM characters at the current position
@@ -1437,6 +1460,7 @@ void	term_repeat (unsigned char c, int rep)
 		while (rep--)
 			putchar_x (c);
 }
+#endif
 
 /*
  * Scroll the screen N lines between lines TOP and BOT.
